@@ -34,7 +34,7 @@ from libcloud.common.openstack_identity import (OpenStackServiceCatalog,
 try:
     import simplejson as json
 except ImportError:
-    import json
+    import json  # type: ignore
 
 AUTH_API_VERSION = '1.1'
 
@@ -109,6 +109,15 @@ class OpenStackBaseConnection(ConnectionUserAndKey):
                            default tenant if none is provided.
     :type ex_tenant_name: ``str``
 
+    :param ex_tenant_domain_id: When authenticating, provide this tenant
+                                domain id to the identity service.
+                                A scoped token will be returned.
+                                Some cloud providers require the tenant
+                                domain id to be provided at authentication
+                                time. Others will use a default tenant
+                                domain id if none is provided.
+    :type ex_tenant_domain_id: ``str``
+
     :param ex_force_service_type: Service type to use when selecting an
                                   service. If not specified, a provider
                                   specific default will be used.
@@ -125,8 +134,8 @@ class OpenStackBaseConnection(ConnectionUserAndKey):
     :type ex_force_service_region: ``str``
     """
 
-    auth_url = None
-    auth_token = None
+    auth_url = None  # type: str
+    auth_token = None  # type: str
     auth_token_expires = None
     auth_user_info = None
     service_catalog = None
@@ -145,6 +154,7 @@ class OpenStackBaseConnection(ConnectionUserAndKey):
                  ex_token_scope=OpenStackIdentityTokenScope.PROJECT,
                  ex_domain_name='Default',
                  ex_tenant_name=None,
+                 ex_tenant_domain_id='default',
                  ex_force_service_type=None,
                  ex_force_service_name=None,
                  ex_force_service_region=None,
@@ -156,12 +166,14 @@ class OpenStackBaseConnection(ConnectionUserAndKey):
         if ex_force_auth_version:
             self._auth_version = ex_force_auth_version
 
+        self.base_url = ex_force_base_url
         self._ex_force_base_url = ex_force_base_url
         self._ex_force_auth_url = ex_force_auth_url
         self._ex_force_auth_token = ex_force_auth_token
         self._ex_token_scope = ex_token_scope
         self._ex_domain_name = ex_domain_name
         self._ex_tenant_name = ex_tenant_name
+        self._ex_tenant_domain_id = ex_tenant_domain_id
         self._ex_force_service_type = ex_force_service_type
         self._ex_force_service_name = ex_force_service_name
         self._ex_force_service_region = ex_force_service_region
@@ -198,9 +210,11 @@ class OpenStackBaseConnection(ConnectionUserAndKey):
                             user_id=self.user_id,
                             key=self.key,
                             tenant_name=self._ex_tenant_name,
+                            tenant_domain_id=self._ex_tenant_domain_id,
                             domain_name=self._ex_domain_name,
                             token_scope=self._ex_token_scope,
                             timeout=self.timeout,
+                            proxy_url=self.proxy_url,
                             parent_conn=self)
 
         return self._osa
@@ -345,7 +359,7 @@ class OpenStackResponse(Response):
 
     def success(self):
         i = int(self.status)
-        return i >= 200 and i <= 299
+        return 200 <= i <= 299
 
     def has_content_type(self, content_type):
         content_type_value = self.headers.get('content-type') or ''
@@ -359,7 +373,7 @@ class OpenStackResponse(Response):
         if self.has_content_type('application/xml'):
             try:
                 return ET.XML(self.body)
-            except:
+            except Exception:
                 raise MalformedResponseError(
                     'Failed to parse XML',
                     body=self.body,
@@ -368,7 +382,7 @@ class OpenStackResponse(Response):
         elif self.has_content_type('application/json'):
             try:
                 return json.loads(self.body)
-            except:
+            except Exception:
                 raise MalformedResponseError(
                     'Failed to parse JSON',
                     body=self.body,
@@ -377,7 +391,6 @@ class OpenStackResponse(Response):
             return self.body
 
     def parse_error(self):
-        text = None
         body = self.parse_body()
 
         if self.has_content_type('application/xml'):
@@ -390,7 +403,8 @@ class OpenStackResponse(Response):
             driver = self.connection.driver
             key_pair_name = context.get('key_pair_name', None)
 
-            if len(values) > 0 and values[0]['code'] == 404 and key_pair_name:
+            if len(values) > 0 and 'code' in values[0] and \
+                    values[0]['code'] == 404 and key_pair_name:
                 raise KeyPairDoesNotExistError(name=key_pair_name,
                                                driver=driver)
             elif len(values) > 0 and 'message' in values[0]:
@@ -418,6 +432,7 @@ class OpenStackDriverMixin(object):
                  ex_token_scope=OpenStackIdentityTokenScope.PROJECT,
                  ex_domain_name='Default',
                  ex_tenant_name=None,
+                 ex_tenant_domain_id='default',
                  ex_force_service_type=None,
                  ex_force_service_name=None,
                  ex_force_service_region=None, *args, **kwargs):
@@ -428,12 +443,14 @@ class OpenStackDriverMixin(object):
         self._ex_token_scope = ex_token_scope
         self._ex_domain_name = ex_domain_name
         self._ex_tenant_name = ex_tenant_name
+        self._ex_tenant_domain_id = ex_tenant_domain_id
         self._ex_force_service_type = ex_force_service_type
         self._ex_force_service_name = ex_force_service_name
         self._ex_force_service_region = ex_force_service_region
 
     def openstack_connection_kwargs(self):
         """
+        Returns certain ``ex_*`` parameters for this connection.
 
         :rtype: ``dict``
         """
@@ -452,6 +469,8 @@ class OpenStackDriverMixin(object):
             rv['ex_domain_name'] = self._ex_domain_name
         if self._ex_tenant_name:
             rv['ex_tenant_name'] = self._ex_tenant_name
+        if self._ex_tenant_domain_id:
+            rv['ex_tenant_domain_id'] = self._ex_tenant_domain_id
         if self._ex_force_service_type:
             rv['ex_force_service_type'] = self._ex_force_service_type
         if self._ex_force_service_name:
