@@ -18,7 +18,6 @@ Common / shared code for handling authentication against OpenStack identity
 service (Keystone).
 """
 
-import sys
 import datetime
 
 from libcloud.utils.py3 import httplib
@@ -32,7 +31,7 @@ from libcloud.compute.types import (LibcloudError, InvalidCredsError,
 try:
     import simplejson as json
 except ImportError:
-    import json
+    import json  # type: ignore
 
 AUTH_API_VERSION = '1.1'
 
@@ -553,7 +552,7 @@ class OpenStackAuthResponse(Response):
         if content_type == 'application/json':
             try:
                 data = json.loads(self.body)
-            except:
+            except Exception:
                 driver = OpenStackIdentityConnection
                 raise MalformedResponseError('Failed to parse JSON',
                                              body=self.body,
@@ -574,16 +573,17 @@ class OpenStackIdentityConnection(ConnectionUserAndKey):
     """
     responseCls = OpenStackAuthResponse
     timeout = None
-    auth_version = None
+    auth_version = None  # type: str
 
     def __init__(self, auth_url, user_id, key, tenant_name=None,
-                 domain_name='Default',
+                 tenant_domain_id='default', domain_name='Default',
                  token_scope=OpenStackIdentityTokenScope.PROJECT,
-                 timeout=None, parent_conn=None):
+                 timeout=None, proxy_url=None, parent_conn=None):
         super(OpenStackIdentityConnection, self).__init__(user_id=user_id,
                                                           key=key,
                                                           url=auth_url,
-                                                          timeout=timeout)
+                                                          timeout=timeout,
+                                                          proxy_url=proxy_url)
 
         self.parent_conn = parent_conn
 
@@ -817,8 +817,7 @@ class OpenStackIdentity_1_1_Connection(OpenStackIdentityConnection):
         else:
             try:
                 body = json.loads(resp.body)
-            except Exception:
-                e = sys.exc_info()[1]
+            except Exception as e:
                 raise MalformedResponseError('Failed to parse JSON', e)
 
             try:
@@ -828,8 +827,7 @@ class OpenStackIdentity_1_1_Connection(OpenStackIdentityConnection):
                 self.auth_token_expires = parse_date(expires)
                 self.urls = body['auth']['serviceCatalog']
                 self.auth_user_info = None
-            except KeyError:
-                e = sys.exc_info()[1]
+            except KeyError as e:
                 raise MalformedResponseError('Auth JSON response is \
                                              missing required elements', e)
 
@@ -902,8 +900,7 @@ class OpenStackIdentity_2_0_Connection(OpenStackIdentityConnection):
                 self.auth_token_expires = parse_date(expires)
                 self.urls = access['serviceCatalog']
                 self.auth_user_info = access.get('user', {})
-            except KeyError:
-                e = sys.exc_info()[1]
+            except KeyError as e:
                 raise MalformedResponseError('Auth JSON response is \
                                              missing required elements', e)
 
@@ -934,9 +931,9 @@ class OpenStackIdentity_3_0_Connection(OpenStackIdentityConnection):
     ]
 
     def __init__(self, auth_url, user_id, key, tenant_name=None,
-                 domain_name='Default',
+                 domain_name='Default', tenant_domain_id='default',
                  token_scope=OpenStackIdentityTokenScope.PROJECT,
-                 timeout=None, parent_conn=None):
+                 timeout=None, proxy_url=None, parent_conn=None):
         """
         :param tenant_name: Name of the project this user belongs to. Note:
                             When token_scope is set to project, this argument
@@ -960,6 +957,7 @@ class OpenStackIdentity_3_0_Connection(OpenStackIdentityConnection):
                              domain_name=domain_name,
                              token_scope=token_scope,
                              timeout=timeout,
+                             proxy_url=proxy_url,
                              parent_conn=parent_conn)
 
         if self.token_scope not in self.VALID_TOKEN_SCOPES:
@@ -975,6 +973,7 @@ class OpenStackIdentity_3_0_Connection(OpenStackIdentityConnection):
             raise ValueError('Must provide domain_name argument')
 
         self.auth_user_roles = None
+        self.tenant_domain_id = tenant_domain_id
 
     def authenticate(self, force=False):
         """
@@ -1005,7 +1004,7 @@ class OpenStackIdentity_3_0_Connection(OpenStackIdentityConnection):
             data['auth']['scope'] = {
                 'project': {
                     'domain': {
-                        'name': self.domain_name
+                        'id': self.tenant_domain_id
                     },
                     'name': self.tenant_name
                 }
@@ -1036,14 +1035,12 @@ class OpenStackIdentity_3_0_Connection(OpenStackIdentityConnection):
 
             try:
                 body = json.loads(response.body)
-            except Exception:
-                e = sys.exc_info()[1]
+            except Exception as e:
                 raise MalformedResponseError('Failed to parse JSON', e)
 
             try:
                 roles = self._to_roles(body['token']['roles'])
-            except Exception:
-                e = sys.exc_info()[1]
+            except Exception as e:
                 roles = []
 
             try:
@@ -1055,8 +1052,7 @@ class OpenStackIdentity_3_0_Connection(OpenStackIdentityConnection):
                 self.urls = body['token'].get('catalog', None)
                 self.auth_user_info = None
                 self.auth_user_roles = roles
-            except KeyError:
-                e = sys.exc_info()[1]
+            except KeyError as e:
                 raise MalformedResponseError('Auth JSON response is \
                                              missing required elements', e)
             body = 'code: %s body:%s' % (response.status, response.body)
@@ -1130,6 +1126,20 @@ class OpenStackIdentity_3_0_Connection(OpenStackIdentityConnection):
         result = self._to_domain(data=response.object['domain'])
         return result
 
+    def get_user(self, user_id):
+        """
+        Get a user account by ID.
+
+        :param user_id: User's id.
+        :type name: ``str``
+
+        :return: Located user.
+        :rtype: :class:`.OpenStackIdentityUser`
+        """
+        response = self.authenticated_request('/v3/users/%s' % user_id)
+        user = self._to_user(data=response.object['user'])
+        return user
+
     def list_user_projects(self, user):
         """
         Retrieve all the projects user belongs to.
@@ -1366,10 +1376,10 @@ class OpenStackIdentity_3_0_Connection(OpenStackIdentityConnection):
         user = OpenStackIdentityUser(id=data['id'],
                                      domain_id=data['domain_id'],
                                      name=data['name'],
-                                     email=data['email'],
+                                     email=data.get('email'),
                                      description=data.get('description',
                                                           None),
-                                     enabled=data['enabled'])
+                                     enabled=data.get('enabled'))
         return user
 
     def _to_roles(self, data):
@@ -1402,8 +1412,9 @@ class OpenStackIdentity_3_0_Connection_OIDC_access_token(
     The protocol name required to get the full path
     must be set in the self.tenant_name attribute.
 
-    The user must be scoped to the first project accessible with the
-    specified access token (usually there are only one)
+    The self.domain_name attribute can be used either to select the
+    domain name in case of domain scoped token or to select the project
+    name in case of project scoped token
     """
 
     responseCls = OpenStackAuthResponse
@@ -1418,7 +1429,6 @@ class OpenStackIdentity_3_0_Connection_OIDC_access_token(
             return self
 
         subject_token = self._get_unscoped_token_from_oidc_token()
-        project_id = self._get_project_id(token=subject_token)
 
         data = {
             'auth': {
@@ -1433,6 +1443,7 @@ class OpenStackIdentity_3_0_Connection_OIDC_access_token(
 
         if self.token_scope == OpenStackIdentityTokenScope.PROJECT:
             # Scope token to project (tenant)
+            project_id = self._get_project_id(token=subject_token)
             data['auth']['scope'] = {
                 'project': {
                     'id': project_id
@@ -1464,14 +1475,12 @@ class OpenStackIdentity_3_0_Connection_OIDC_access_token(
 
             try:
                 body = json.loads(response.body)
-            except Exception:
-                e = sys.exc_info()[1]
+            except Exception as e:
                 raise MalformedResponseError('Failed to parse JSON', e)
 
             try:
                 roles = self._to_roles(body['token']['roles'])
-            except Exception:
-                e = sys.exc_info()[1]
+            except Exception as e:
                 roles = []
 
             try:
@@ -1483,8 +1492,7 @@ class OpenStackIdentity_3_0_Connection_OIDC_access_token(
                 self.urls = body['token'].get('catalog', None)
                 self.auth_user_info = None
                 self.auth_user_roles = roles
-            except KeyError:
-                e = sys.exc_info()[1]
+            except KeyError as e:
                 raise MalformedResponseError('Auth JSON response is \
                                              missing required elements', e)
             body = 'code: %s body:%s' % (response.status, response.body)
@@ -1518,31 +1526,51 @@ class OpenStackIdentity_3_0_Connection_OIDC_access_token(
                                              driver=self.driver)
         else:
             raise MalformedResponseError('Malformed response',
-                                         driver=self.driver)
+                                         driver=self.driver,
+                                         body=response.body)
 
     def _get_project_id(self, token):
         """
         Get the first project ID accessible with the specified access token
         """
-        path = '/v3/OS-FEDERATION/projects'
+        # Try new path first (from ver 1.1)
+        path = '/v3/auth/projects'
         response = self.request(path,
                                 headers={'Content-Type': 'application/json',
                                          'X-Auth-Token': token},
                                 method='GET')
 
+        if response.status not in [httplib.UNAUTHORIZED, httplib.OK,
+                                   httplib.CREATED]:
+            # In case of error try old one
+            path = '/v3/OS-FEDERATION/projects'
+            response = self.request(path,
+                                    headers={'Content-Type':
+                                             'application/json',
+                                             'X-Auth-Token': token},
+                                    method='GET')
+
         if response.status == httplib.UNAUTHORIZED:
             # Invalid credentials
             raise InvalidCredsError()
         elif response.status in [httplib.OK, httplib.CREATED]:
             try:
                 body = json.loads(response.body)
-                return body["projects"][0]["id"]
-            except Exception:
-                e = sys.exc_info()[1]
+                # We use domain_name in both cases of the scoped tokens
+                # as we have used tenant as the protocol
+                if self.domain_name and self.domain_name != 'Default':
+                    for project in body['projects']:
+                        if project['name'] == self.domain_name:
+                            return project['id']
+                    raise ValueError('Project %s not found' % self.domain_name)
+                else:
+                    return body['projects'][0]['id']
+            except Exception as e:
                 raise MalformedResponseError('Failed to parse JSON', e)
         else:
             raise MalformedResponseError('Malformed response',
-                                         driver=self.driver)
+                                         driver=self.driver,
+                                         body=response.body)
 
 
 class OpenStackIdentity_2_0_Connection_VOMS(OpenStackIdentityConnection,
@@ -1559,9 +1587,10 @@ class OpenStackIdentity_2_0_Connection_VOMS(OpenStackIdentityConnection,
     def __init__(self, auth_url, user_id, key, tenant_name=None,
                  domain_name='Default',
                  token_scope=OpenStackIdentityTokenScope.PROJECT,
-                 timeout=None, parent_conn=None):
+                 timeout=None, proxy_url=None, parent_conn=None):
         CertificateConnection.__init__(self, cert_file=key,
                                        url=auth_url,
+                                       proxy_url=proxy_url,
                                        timeout=timeout)
 
         self.parent_conn = parent_conn
@@ -1578,6 +1607,7 @@ class OpenStackIdentity_2_0_Connection_VOMS(OpenStackIdentityConnection,
         self.domain_name = domain_name
         self.token_scope = token_scope
         self.timeout = timeout
+        self.proxy_url = proxy_url
 
         self.urls = {}
         self.auth_token = None
@@ -1617,12 +1647,12 @@ class OpenStackIdentity_2_0_Connection_VOMS(OpenStackIdentityConnection,
             try:
                 body = json.loads(response.body)
                 return body['access']['token']['id']
-            except Exception:
-                e = sys.exc_info()[1]
+            except Exception as e:
                 raise MalformedResponseError('Failed to parse JSON', e)
         else:
             raise MalformedResponseError('Malformed response',
-                                         driver=self.driver)
+                                         driver=self.driver,
+                                         body=response.body)
 
     def _get_tenant_name(self, token):
         """
@@ -1640,12 +1670,12 @@ class OpenStackIdentity_2_0_Connection_VOMS(OpenStackIdentityConnection,
             try:
                 body = json.loads(response.body)
                 return body["tenants"][0]["name"]
-            except Exception:
-                e = sys.exc_info()[1]
+            except Exception as e:
                 raise MalformedResponseError('Failed to parse JSON', e)
         else:
             raise MalformedResponseError('Malformed response',
-                                         driver=self.driver)
+                                         driver=self.driver,
+                                         body=response.body)
 
     def _authenticate_2_0_with_body(self, reqbody):
         resp = self.request('/v2.0/tokens', data=reqbody,
@@ -1670,8 +1700,7 @@ class OpenStackIdentity_2_0_Connection_VOMS(OpenStackIdentityConnection,
                 self.auth_token_expires = parse_date(expires)
                 self.urls = access['serviceCatalog']
                 self.auth_user_info = access.get('user', {})
-            except KeyError:
-                e = sys.exc_info()[1]
+            except KeyError as e:
                 raise MalformedResponseError('Auth JSON response is \
                                              missing required elements', e)
 
