@@ -20,6 +20,8 @@ Provides base classes for working with storage
 # Backward compatibility for Python 2.5
 from __future__ import with_statement
 
+from typing import Type
+
 import os.path                          # pylint: disable-msg=W0404
 import hashlib
 from os.path import join as pjoin
@@ -29,6 +31,7 @@ from libcloud.utils.py3 import b
 
 import libcloud.utils.files
 from libcloud.common.types import LibcloudError
+from libcloud.common.base import Connection
 from libcloud.common.base import ConnectionUserAndKey, BaseDriver
 from libcloud.storage.types import ObjectDoesNotExistError
 
@@ -180,14 +183,14 @@ class StorageDriver(BaseDriver):
     A base StorageDriver to derive from.
     """
 
-    connectionCls = ConnectionUserAndKey
-    name = None
-    hash_type = 'md5'
-    supports_chunked_encoding = False
+    connectionCls = ConnectionUserAndKey  # type: Type[Connection]
+    name = None  # type: str
+    hash_type = 'md5'  # type: str
+    supports_chunked_encoding = False  # type: bool
 
     # When strict mode is used, exception will be thrown if no content type is
     # provided and none can be detected when uploading an object
-    strict_mode = False
+    strict_mode = False  # type: bool
 
     def iterate_containers(self):
         """
@@ -221,13 +224,16 @@ class StorageDriver(BaseDriver):
         raise NotImplementedError(
             'iterate_container_objects not implemented for this driver')
 
-    def list_container_objects(self, container):
+    def list_container_objects(self, container, ex_prefix=None):
         """
         Return a list of objects for the given container.
 
         :param container: Container instance.
         :type container: :class:`Container`
 
+        :param ex_prefix: Filter objects starting with a prefix.
+        :type  ex_prefix: ``str``
+
         :return: A list of Object instances.
         :rtype: ``list`` of :class:`Object`
         """
@@ -642,22 +648,32 @@ class StorageDriver(BaseDriver):
 
     def _hash_buffered_stream(self, stream, hasher, blocksize=65536):
         total_len = 0
-        if hasattr(stream, '__next__'):
-            data = libcloud.utils.files.exhaust_iterator(iterator=stream)
-            hasher.update(b(data))
-            total_len = len(data)
+
+        if hasattr(stream, '__next__') or hasattr(stream, 'next'):
+            # Ensure we start from the begining of a stream in case stream is
+            # not at the beginning
+            if hasattr(stream, 'seek'):
+                stream.seek(0)
+
+            for chunk in libcloud.utils.files.read_in_chunks(iterator=stream):
+                hasher.update(b(chunk))
+                total_len += len(chunk)
+
             return (hasher.hexdigest(), total_len)
+
         if not hasattr(stream, '__exit__'):
             for s in stream:
                 hasher.update(s)
                 total_len = total_len + len(s)
             return (hasher.hexdigest(), total_len)
+
         with stream:
             buf = stream.read(blocksize)
             while len(buf) > 0:
                 total_len = total_len + len(buf)
                 hasher.update(buf)
                 buf = stream.read(blocksize)
+
         return (hasher.hexdigest(), total_len)
 
     def _get_hash_function(self):
