@@ -16,10 +16,10 @@
 import base64
 import hmac
 import time
-
 from hashlib import sha1
 
 import libcloud.utils.py3
+
 try:
     if libcloud.utils.py3.DEFAULT_LXML:
         from lxml.etree import Element, SubElement
@@ -48,7 +48,6 @@ from libcloud.storage.types import ContainerDoesNotExistError
 from libcloud.storage.types import ObjectDoesNotExistError
 from libcloud.storage.types import ObjectHashMismatchError
 
-
 # How long before the token expires
 EXPIRATION_SECONDS = 15 * 60
 
@@ -58,9 +57,11 @@ S3_US_WEST_HOST = 's3-us-west-1.amazonaws.com'
 S3_US_WEST_OREGON_HOST = 's3-us-west-2.amazonaws.com'
 S3_US_GOV_WEST_HOST = 's3-us-gov-west-1.amazonaws.com'
 S3_CN_NORTH_HOST = 's3.cn-north-1.amazonaws.com.cn'
+S3_CN_NORTHWEST_HOST = 's3.cn-northwest-1.amazonaws.com.cn'
 S3_EU_WEST_HOST = 's3-eu-west-1.amazonaws.com'
 S3_EU_WEST2_HOST = 's3-eu-west-2.amazonaws.com'
 S3_EU_CENTRAL_HOST = 's3-eu-central-1.amazonaws.com'
+S3_EU_NORTH1_HOST = 's3-eu-north-1.amazonaws.com'
 S3_AP_SOUTH_HOST = 's3-ap-south-1.amazonaws.com'
 S3_AP_SOUTHEAST_HOST = 's3-ap-southeast-1.amazonaws.com'
 S3_AP_SOUTHEAST2_HOST = 's3-ap-southeast-2.amazonaws.com'
@@ -71,6 +72,32 @@ S3_SA_EAST_HOST = 's3-sa-east-1.amazonaws.com'
 S3_SA_SOUTHEAST2_HOST = 's3-sa-east-2.amazonaws.com'
 S3_CA_CENTRAL_HOST = 's3-ca-central-1.amazonaws.com'
 
+# Maps AWS region name to connection hostname
+REGION_TO_HOST_MAP = {
+    'us-east-1': S3_US_STANDARD_HOST,
+    'us-east-2': S3_US_EAST2_HOST,
+    'us-west-1': S3_US_WEST_HOST,
+    'us-west-2': S3_US_WEST_OREGON_HOST,
+    'us-gov-west-1': S3_US_GOV_WEST_HOST,
+    'cn-north-1': S3_CN_NORTH_HOST,
+    'cn-northwest-1': S3_CN_NORTHWEST_HOST,
+    'eu-west-1': S3_EU_WEST_HOST,
+    'eu-west-2': S3_EU_WEST2_HOST,
+    'eu-west-3': 's3.eu-west-3.amazonaws.com',
+    'eu-north-1': 's3.eu-north-1.amazonaws.com',
+    'eu-central-1': S3_EU_CENTRAL_HOST,
+    'ap-south-1': S3_AP_SOUTH_HOST,
+    'ap-southeast-1': S3_AP_SOUTHEAST_HOST,
+    'ap-southeast-2': S3_AP_SOUTHEAST2_HOST,
+    'ap-northeast-1': S3_AP_NORTHEAST1_HOST,
+    'ap-northeast-2': S3_AP_NORTHEAST2_HOST,
+    'ap-northeast-3': 's3.ap-northeast-3.amazonaws.com',
+    'sa-east-1': S3_SA_EAST_HOST,
+    'sa-east-2': S3_SA_SOUTHEAST2_HOST,
+    'ca-central-1': S3_CA_CENTRAL_HOST,
+    'me-south-1': 's3.me-south-1.amazonaws.com'
+}
+
 API_VERSION = '2006-03-01'
 NAMESPACE = 'http://s3.amazonaws.com/doc/%s/' % (API_VERSION)
 
@@ -89,14 +116,18 @@ class S3Response(AWSBaseResponse):
 
     def success(self):
         i = int(self.status)
-        return i >= 200 and i <= 299 or i in self.valid_response_codes
+        return 200 <= i <= 299 or i in self.valid_response_codes
 
     def parse_error(self):
         if self.status in [httplib.UNAUTHORIZED, httplib.FORBIDDEN]:
             raise InvalidCredsError(self.body)
         elif self.status == httplib.MOVED_PERMANENTLY:
-            raise LibcloudError('This bucket is located in a different ' +
-                                'region. Please use the correct driver.',
+            bucket_region = self.headers.get('x-amz-bucket-region', None)
+            used_region = self.connection.driver.region
+            raise LibcloudError('This bucket is located in a different '
+                                'region. Please use the correct driver. '
+                                'Bucket region "%s", used region "%s".' %
+                                (bucket_region, used_region),
                                 driver=S3StorageDriver)
         raise LibcloudError('Unknown error. Status code: %d' % (self.status),
                             driver=S3StorageDriver)
@@ -170,6 +201,7 @@ class BaseS3Connection(ConnectionUserAndKey):
         return params
 
     def pre_connect_hook(self, params, headers):
+        # pylint: disable=no-member
         params['Signature'] = self.get_auth_signature(
             method=self.method, headers=headers, params=params,
             expires=params['Expires'], secret_key=self.key, path=self.action,
@@ -727,7 +759,6 @@ class BaseS3StorageDriver(StorageDriver):
             # pylint: disable=maybe-no-member
             for node in body.findall(fixxpath(xpath='Upload',
                                               namespace=self.namespace)):
-
                 initiator = node.find(fixxpath(xpath='Initiator',
                                                namespace=self.namespace))
                 owner = node.find(fixxpath(xpath='Owner',
@@ -869,6 +900,10 @@ class BaseS3StorageDriver(StorageDriver):
         meta_data = extra.get('meta_data', None)
         acl = extra.get('acl', None)
 
+        if not content_type:
+            content_type, _ = libcloud.utils.files.guess_file_mime_type(
+                object_name)
+
         if content_type:
             headers['Content-Type'] = content_type
 
@@ -997,10 +1032,37 @@ class BaseS3StorageDriver(StorageDriver):
 
 
 class S3StorageDriver(AWSDriver, BaseS3StorageDriver):
-    name = 'Amazon S3 (us-east-1)'
+    name = 'Amazon S3'
     connectionCls = S3SignatureV4Connection
     region_name = 'us-east-1'
 
+    def __init__(self, key, secret=None, secure=True, host=None, port=None,
+                 region=None, token=None, **kwargs):
+        # Here for backward compatibility for old and deprecated driver class
+        # per region approach
+        if hasattr(self, 'region_name') and not region:
+            region = self.region_name  # pylint: disable=no-member
+
+        self.region_name = region
+
+        if region and region not in REGION_TO_HOST_MAP.keys():
+            raise ValueError('Invalid or unsupported region: %s' % (region))
+
+        self.name = 'Amazon S3 (%s)' % (region)
+
+        if host is None:
+            host = REGION_TO_HOST_MAP[region]
+
+        super(S3StorageDriver, self).__init__(key=key, secret=secret,
+                                              secure=secure, host=host,
+                                              port=port,
+                                              region=region, token=token,
+                                              **kwargs)
+
+    @classmethod
+    def list_regions(self):
+        return REGION_TO_HOST_MAP.keys()
+
 
 class S3USEast2Connection(S3SignatureV4Connection):
     host = S3_US_EAST2_HOST
@@ -1046,6 +1108,17 @@ class S3USGovWestStorageDriver(S3StorageDriver):
     region_name = 'us-gov-west-1'
 
 
+class S3CNNorthWestConnection(S3SignatureV4Connection):
+    host = S3_CN_NORTHWEST_HOST
+
+
+class S3CNNorthWestStorageDriver(S3StorageDriver):
+    name = 'Amazon S3 (cn-northwest-1)'
+    connectionCls = S3CNNorthWestConnection
+    ex_location_name = 'cn-northwest-1'
+    region_name = 'cn-northwest-1'
+
+
 class S3CNNorthConnection(S3SignatureV4Connection):
     host = S3_CN_NORTH_HOST
 
@@ -1094,6 +1167,17 @@ class S3APSEConnection(S3SignatureV4Connection):
     host = S3_AP_SOUTHEAST_HOST
 
 
+class S3EUNorth1Connection(S3SignatureV4Connection):
+    host = S3_EU_NORTH1_HOST
+
+
+class S3EUNorth1StorageDriver(S3StorageDriver):
+    name = 'Amazon S3 (eu-north-1)'
+    connectionCls = S3EUNorth1Connection
+    ex_location_name = 'eu-north-1'
+    region_name = 'eu-north-1'
+
+
 class S3APSEStorageDriver(S3StorageDriver):
     name = 'Amazon S3 (ap-southeast-1)'
     connectionCls = S3APSEConnection
@@ -1115,6 +1199,7 @@ class S3APSE2StorageDriver(S3StorageDriver):
 class S3APNE1Connection(S3SignatureV4Connection):
     host = S3_AP_NORTHEAST1_HOST
 
+
 S3APNEConnection = S3APNE1Connection
 
 
@@ -1124,6 +1209,7 @@ class S3APNE1StorageDriver(S3StorageDriver):
     ex_location_name = 'ap-northeast-1'
     region_name = 'ap-northeast-1'
 
+
 S3APNEStorageDriver = S3APNE1StorageDriver
 
 
