@@ -24,6 +24,7 @@ except ImportError:
 from mock import Mock
 
 from libcloud.utils.py3 import httplib
+from libcloud.utils.py3 import assertRaisesRegex
 from libcloud.common.openstack import OpenStackBaseConnection
 from libcloud.common.openstack_identity import AUTH_TOKEN_EXPIRES_GRACE_SECONDS
 from libcloud.common.openstack_identity import get_class_for_auth_version
@@ -93,7 +94,7 @@ class OpenStackIdentityConnectionTestCase(unittest.TestCase):
 
                 try:
                     osa = osa.authenticate()
-                except:
+                except Exception:
                     pass
 
                 if (should_append_default_path == APPEND):
@@ -123,8 +124,8 @@ class OpenStackIdentityConnectionTestCase(unittest.TestCase):
                       parent_conn=connection)
 
             self.assertEqual(osa.urls, {})
-            self.assertEqual(osa.auth_token, None)
-            self.assertEqual(osa.auth_user_info, None)
+            self.assertIsNone(osa.auth_token)
+            self.assertIsNone(osa.auth_user_info)
             osa = osa.authenticate()
 
             self.assertTrue(len(osa.urls) >= 1)
@@ -231,8 +232,11 @@ class OpenStackIdentity_2_0_ConnectionTests(unittest.TestCase):
         self.auth_instance = OpenStackIdentity_2_0_Connection(auth_url='http://none',
                                                               user_id='test',
                                                               key='test',
-                                                              tenant_name='test')
+                                                              tenant_name='test',
+                                                              proxy_url='http://proxy:8080',
+                                                              timeout=10)
         self.auth_instance.auth_token = 'mock'
+        self.assertEqual(self.auth_instance.proxy_url, 'http://proxy:8080')
 
     def test_list_projects(self):
         result = self.auth_instance.list_projects()
@@ -252,37 +256,40 @@ class OpenStackIdentity_3_0_ConnectionTests(unittest.TestCase):
         self.auth_instance = OpenStackIdentity_3_0_Connection(auth_url='http://none',
                                                               user_id='test',
                                                               key='test',
-                                                              tenant_name='test')
+                                                              tenant_name='test',
+                                                              proxy_url='http://proxy:8080',
+                                                              timeout=10)
         self.auth_instance.auth_token = 'mock'
+        self.assertEqual(self.auth_instance.proxy_url, 'http://proxy:8080')
 
     def test_token_scope_argument(self):
         # Invalid token_scope value
         expected_msg = 'Invalid value for "token_scope" argument: foo'
-        self.assertRaisesRegexp(ValueError, expected_msg,
-                                OpenStackIdentity_3_0_Connection,
-                                auth_url='http://none',
-                                user_id='test',
-                                key='test',
-                                token_scope='foo')
+        assertRaisesRegex(self, ValueError, expected_msg,
+                          OpenStackIdentity_3_0_Connection,
+                          auth_url='http://none',
+                          user_id='test',
+                          key='test',
+                          token_scope='foo')
 
         # Missing tenant_name
         expected_msg = 'Must provide tenant_name and domain_name argument'
-        self.assertRaisesRegexp(ValueError, expected_msg,
-                                OpenStackIdentity_3_0_Connection,
-                                auth_url='http://none',
-                                user_id='test',
-                                key='test',
-                                token_scope='project')
+        assertRaisesRegex(self, ValueError, expected_msg,
+                          OpenStackIdentity_3_0_Connection,
+                          auth_url='http://none',
+                          user_id='test',
+                          key='test',
+                          token_scope='project')
 
         # Missing domain_name
         expected_msg = 'Must provide domain_name argument'
-        self.assertRaisesRegexp(ValueError, expected_msg,
-                                OpenStackIdentity_3_0_Connection,
-                                auth_url='http://none',
-                                user_id='test',
-                                key='test',
-                                token_scope='domain',
-                                domain_name=None)
+        assertRaisesRegex(self, ValueError, expected_msg,
+                          OpenStackIdentity_3_0_Connection,
+                          auth_url='http://none',
+                          user_id='test',
+                          key='test',
+                          token_scope='domain',
+                          domain_name=None)
 
         # Scope to project all ok
         OpenStackIdentity_3_0_Connection(auth_url='http://none',
@@ -305,8 +312,12 @@ class OpenStackIdentity_3_0_ConnectionTests(unittest.TestCase):
                                                 key='test_key',
                                                 token_scope='project',
                                                 tenant_name="test_tenant",
-                                                domain_name='test_domain')
+                                                tenant_domain_id="test_tenant_domain_id",
+                                                domain_name='test_domain',
+                                                proxy_url='http://proxy:8080',
+                                                timeout=10)
         auth.authenticate()
+        self.assertEqual(auth.proxy_url, 'http://proxy:8080')
 
     def test_list_supported_versions(self):
         OpenStackIdentity_3_0_MockHttp.type = 'v3'
@@ -366,6 +377,25 @@ class OpenStackIdentity_3_0_ConnectionTests(unittest.TestCase):
         domain = self.auth_instance.get_domain(domain_id='default')
         self.assertEqual(domain.name, 'Default')
 
+    def test_get_user(self):
+        user = self.auth_instance.get_user(user_id='a')
+        self.assertEqual(user.id, 'a')
+        self.assertEqual(user.domain_id, 'default')
+        self.assertEqual(user.enabled, True)
+        self.assertEqual(user.email, 'openstack-test@localhost')
+
+    def test_get_user_without_email(self):
+        user = self.auth_instance.get_user(user_id='b')
+        self.assertEqual(user.id, 'b')
+        self.assertEqual(user.name, 'userwithoutemail')
+        self.assertIsNone(user.email)
+
+    def test_get_user_without_enabled(self):
+        user = self.auth_instance.get_user(user_id='c')
+        self.assertEqual(user.id, 'c')
+        self.assertEqual(user.name, 'userwithoutenabled')
+        self.assertIsNone(user.enabled)
+
     def test_create_user(self):
         user = self.auth_instance.create_user(email='test2@localhost', password='test1',
                                               name='test2', domain_id='default')
@@ -424,6 +454,32 @@ class OpenStackIdentity_3_0_ConnectionTests(unittest.TestCase):
         self.assertTrue(result)
 
 
+class OpenStackIdentity_3_0_Connection_OIDC_access_token_federation_projectsTests(
+        unittest.TestCase):
+    def setUp(self):
+        mock_cls = OpenStackIdentity_3_0_federation_projects_MockHttp
+        mock_cls.type = None
+        OpenStackIdentity_3_0_Connection_OIDC_access_token.conn_class = mock_cls
+
+        self.auth_instance = OpenStackIdentity_3_0_Connection_OIDC_access_token(auth_url='http://none',
+                                                                                user_id='idp',
+                                                                                key='token',
+                                                                                tenant_name='oidc',
+                                                                                proxy_url='http://proxy:8080',
+                                                                                timeout=10)
+        self.auth_instance.auth_token = 'mock'
+
+    def test_authenticate(self):
+        auth = OpenStackIdentity_3_0_Connection_OIDC_access_token(auth_url='http://none',
+                                                                  user_id='idp',
+                                                                  key='token',
+                                                                  token_scope='project',
+                                                                  tenant_name="oidc",
+                                                                  proxy_url='http://proxy:8080',
+                                                                  timeout=10)
+        auth.authenticate()
+
+
 class OpenStackIdentity_3_0_Connection_OIDC_access_tokenTests(
         unittest.TestCase):
     def setUp(self):
@@ -435,7 +491,7 @@ class OpenStackIdentity_3_0_Connection_OIDC_access_tokenTests(
                                                                                 user_id='idp',
                                                                                 key='token',
                                                                                 tenant_name='oidc',
-                                                                                domain_name='test_domain')
+                                                                                domain_name='project_name2')
         self.auth_instance.auth_token = 'mock'
 
     def test_authenticate(self):
@@ -444,7 +500,7 @@ class OpenStackIdentity_3_0_Connection_OIDC_access_tokenTests(
                                                                   key='token',
                                                                   token_scope='project',
                                                                   tenant_name="oidc",
-                                                                  domain_name='test_domain')
+                                                                  domain_name='project_name2')
         auth.authenticate()
 
 
@@ -484,7 +540,7 @@ class OpenStackServiceCatalogTestCase(unittest.TestCase):
 
         entry = [e for e in entries if e.service_type == 'cloudFilesCDN'][0]
         self.assertEqual(entry.service_type, 'cloudFilesCDN')
-        self.assertEqual(entry.service_name, None)
+        self.assertIsNone(entry.service_name)
         self.assertEqual(len(entry.endpoints), 2)
         self.assertEqual(entry.endpoints[0].region, 'ORD')
         self.assertEqual(entry.endpoints[0].url,
@@ -501,13 +557,13 @@ class OpenStackServiceCatalogTestCase(unittest.TestCase):
         catalog = OpenStackServiceCatalog(service_catalog=service_catalog,
                                           auth_version='2.0')
         entries = catalog.get_entries()
-        self.assertEqual(len(entries), 6)
+        self.assertEqual(len(entries), 9)
 
         entry = [e for e in entries if e.service_name == 'cloudServers'][0]
         self.assertEqual(entry.service_type, 'compute')
         self.assertEqual(entry.service_name, 'cloudServers')
         self.assertEqual(len(entry.endpoints), 1)
-        self.assertEqual(entry.endpoints[0].region, None)
+        self.assertIsNone(entry.endpoints[0].region)
         self.assertEqual(entry.endpoints[0].url,
                          'https://servers.api.rackspacecloud.com/v1.0/1337')
         self.assertEqual(entry.endpoints[0].endpoint_type, 'external')
@@ -523,7 +579,7 @@ class OpenStackServiceCatalogTestCase(unittest.TestCase):
         self.assertEqual(len(entries), 6)
         entry = [e for e in entries if e.service_type == 'volume'][0]
         self.assertEqual(entry.service_type, 'volume')
-        self.assertEqual(entry.service_name, None)
+        self.assertIsNone(entry.service_name)
         self.assertEqual(len(entry.endpoints), 3)
         self.assertEqual(entry.endpoints[0].region, 'regionOne')
         self.assertEqual(entry.endpoints[0].endpoint_type, 'external')
@@ -567,8 +623,9 @@ class OpenStackServiceCatalogTestCase(unittest.TestCase):
         catalog = OpenStackServiceCatalog(service_catalog=service_catalog,
                                           auth_version='2.0')
         service_types = catalog.get_service_types()
-        self.assertEqual(service_types, ['compute', 'object-store',
-                                         'rax:object-cdn'])
+        self.assertEqual(service_types, ['compute', 'image', 'network',
+                                         'object-store', 'rax:object-cdn',
+                                         'volumev2'])
 
         service_types = catalog.get_service_types(region='ORD')
         self.assertEqual(service_types, ['rax:object-cdn'])
@@ -582,10 +639,12 @@ class OpenStackServiceCatalogTestCase(unittest.TestCase):
                                           auth_version='2.0')
 
         service_names = catalog.get_service_names()
-        self.assertEqual(service_names, ['cloudFiles', 'cloudFilesCDN',
-                                         'cloudServers',
+        self.assertEqual(service_names, ['cinderv2', 'cloudFiles',
+                                         'cloudFilesCDN', 'cloudServers',
                                          'cloudServersOpenStack',
                                          'cloudServersPreprod',
+                                         'glance',
+                                         'neutron',
                                          'nova'])
 
         service_names = catalog.get_service_names(service_type='compute')
@@ -634,7 +693,7 @@ class OpenStackIdentity_3_0_MockHttp(MockHttp):
             data = json.loads(body)
             if 'password' in data['auth']['identity']:
                 if data['auth']['identity']['password']['user']['domain']['name'] != 'test_domain' or \
-                        data['auth']['scope']['project']['domain']['name'] != 'test_domain':
+                        data['auth']['scope']['project']['domain']['id'] != 'test_tenant_domain_id':
                     status = httplib.UNAUTHORIZED
 
             body = ComputeFileFixtures('openstack').load('_v3__auth.json')
@@ -656,12 +715,30 @@ class OpenStackIdentity_3_0_MockHttp(MockHttp):
         raise NotImplementedError()
 
     def _v3_users_a(self, method, url, body, headers):
+        if method == 'GET':
+            # look up a user
+            body = self.fixtures.load('v3_users_a.json')
+            return (httplib.OK, body, self.json_content_headers, httplib.responses[httplib.OK])
         if method == 'PATCH':
             # enable / disable user
             body = self.fixtures.load('v3_users_a.json')
             return (httplib.OK, body, self.json_content_headers, httplib.responses[httplib.OK])
         raise NotImplementedError()
 
+    def _v3_users_b(self, method, url, body, headers):
+        if method == 'GET':
+            # look up a user
+            body = self.fixtures.load('v3_users_b.json')
+            return (httplib.OK, body, self.json_content_headers, httplib.responses[httplib.OK])
+        raise NotImplementedError()
+
+    def _v3_users_c(self, method, url, body, headers):
+        if method == 'GET':
+            # look up a user
+            body = self.fixtures.load('v3_users_c.json')
+            return (httplib.OK, body, self.json_content_headers, httplib.responses[httplib.OK])
+        raise NotImplementedError()
+
     def _v3_roles(self, method, url, body, headers):
         if method == 'GET':
             body = self.fixtures.load('v3_roles.json')
@@ -722,6 +799,27 @@ class OpenStackIdentity_3_0_MockHttp(MockHttp):
             return (httplib.OK, body, headers, httplib.responses[httplib.OK])
         raise NotImplementedError()
 
+    def _v3_OS_FEDERATION_projects(self, method, url, body, headers):
+        if method == 'GET':
+            # get user projects
+            body = json.dumps({"projects": [{"id": "project_id", "name": "project_name"},
+                                            {"id": "project_id2", "name": "project_name2"}]})
+            return (httplib.OK, body, self.json_content_headers, httplib.responses[httplib.OK])
+        raise NotImplementedError()
+
+    def _v3_auth_projects(self, method, url, body, headers):
+        if method == 'GET':
+            # get user projects
+            body = json.dumps({"projects": [{"id": "project_id", "name": "project_name"},
+                                            {"id": "project_id2", "name": "project_name2"}]})
+            return (httplib.OK, body, self.json_content_headers, httplib.responses[httplib.OK])
+        raise NotImplementedError()
+
+
+class OpenStackIdentity_3_0_federation_projects_MockHttp(OpenStackIdentity_3_0_MockHttp):
+    fixtures = ComputeFileFixtures('openstack_identity/v3')
+    json_content_headers = {'content-type': 'application/json; charset=UTF-8'}
+
     def _v3_OS_FEDERATION_projects(self, method, url, body, headers):
         if method == 'GET':
             # get user projects
@@ -729,6 +827,10 @@ class OpenStackIdentity_3_0_MockHttp(MockHttp):
             return (httplib.OK, body, self.json_content_headers, httplib.responses[httplib.OK])
         raise NotImplementedError()
 
+    def _v3_auth_projects(self, method, url, body, headers):
+        return (httplib.INTERNAL_SERVER_ERROR, body, self.json_content_headers,
+                httplib.responses[httplib.INTERNAL_SERVER_ERROR])
+
 
 class OpenStackIdentity_2_0_Connection_VOMSMockHttp(MockHttp):
     fixtures = ComputeFileFixtures('openstack_identity/v2')
@@ -754,5 +856,6 @@ class OpenStackIdentity_2_0_Connection_VOMSMockHttp(MockHttp):
             return (httplib.OK, body, self.json_content_headers, httplib.responses[httplib.OK])
         raise NotImplementedError()
 
+
 if __name__ == '__main__':
     sys.exit(unittest.main())
