@@ -15,6 +15,8 @@
 
 from __future__ import with_statement
 
+from collections import OrderedDict
+
 import os
 import sys
 from datetime import datetime
@@ -75,10 +77,13 @@ class BaseEC2Tests(LibcloudTestCase):
         unsupported_regions = list()
 
         for region in VALID_EC2_REGIONS:
+            no_pricing = region in ['cn-north-1']
             driver = EC2NodeDriver(*EC2_PARAMS, **{'region': region})
             try:
-                driver.list_sizes()
-            except:
+                sizes = driver.list_sizes()
+                if no_pricing:
+                    self.assertTrue(all([s.price is None for s in sizes]))
+            except Exception:
                 unsupported_regions.append(region)
 
         if unsupported_regions:
@@ -102,14 +107,14 @@ class EC2Tests(LibcloudTestCase, TestCaseMixin):
         token = 'temporary_credentials_token'
         driver = EC2NodeDriver(*EC2_PARAMS, **{'region': self.region, 'token': token})
         self.assertTrue(hasattr(driver, 'token'), 'Driver has no attribute token')
-        self.assertEquals(token, driver.token, "Driver token does not match with provided token")
+        self.assertEqual(token, driver.token, "Driver token does not match with provided token")
 
     def test_driver_with_token_signature_version(self):
         token = 'temporary_credentials_token'
         driver = EC2NodeDriver(*EC2_PARAMS, **{'region': self.region, 'token': token})
         kwargs = driver._ex_connection_class_kwargs()
         self.assertIn('signature_version', kwargs)
-        self.assertEquals('4', kwargs['signature_version'], 'Signature version is not 4 with temporary credentials')
+        self.assertEqual('4', kwargs['signature_version'], 'Signature version is not 4 with temporary credentials')
 
     def test_create_node(self):
         image = NodeImage(id='ami-be3adfd7',
@@ -211,8 +216,7 @@ class EC2Tests(LibcloudTestCase, TestCaseMixin):
             self.driver.create_node(name='foo', image=image, size=size,
                                     ex_mincount='2', ex_maxcount='2',
                                     ex_clienttoken=token)
-        except IdempotentParamError:
-            e = sys.exc_info()[1]
+        except IdempotentParamError as e:
             idem_error = e
         self.assertTrue(idem_error is not None)
 
@@ -326,10 +330,12 @@ class EC2Tests(LibcloudTestCase, TestCaseMixin):
 
     def test_authorize_security_group_ingress(self):
         ranges = ['1.1.1.1/32', '2.2.2.2/32']
-        resp = self.driver.ex_authorize_security_group_ingress('sg-42916629', 22, 22, cidr_ips=ranges)
+        description = "automated authorised IP ingress test"
+        resp = self.driver.ex_authorize_security_group_ingress('sg-42916629', 22, 22, cidr_ips=ranges, description=description)
         self.assertTrue(resp)
         groups = [{'group_id': 'sg-949265ff'}]
-        resp = self.driver.ex_authorize_security_group_ingress('sg-42916629', 22, 23, group_pairs=groups)
+        description = "automated authorised group ingress test"
+        resp = self.driver.ex_authorize_security_group_ingress('sg-42916629', 22, 23, group_pairs=groups, description=description)
         self.assertTrue(resp)
 
     def test_authorize_security_group_egress(self):
@@ -430,39 +436,7 @@ class EC2Tests(LibcloudTestCase, TestCaseMixin):
             self.driver.api_name = api_name
             self.driver.region_name = region_name
             sizes = self.driver.list_sizes()
-
-            ids = [s.id for s in sizes]
-
-            if region_name not in ['ap-south-1']:
-                self.assertTrue('t1.micro' in ids)
-                self.assertTrue('m1.small' in ids)
-                self.assertTrue('m1.large' in ids)
-                self.assertTrue('m1.xlarge' in ids)
-                self.assertTrue('c1.medium' in ids)
-                self.assertTrue('c1.xlarge' in ids)
-                self.assertTrue('m2.xlarge' in ids)
-                self.assertTrue('m2.2xlarge' in ids)
-                self.assertTrue('m2.4xlarge' in ids)
-
-            if region_name == 'us-east-1':
-                self.assertEqual(len(sizes), 70)
-                self.assertTrue('cg1.4xlarge' in ids)
-                self.assertTrue('cc2.8xlarge' in ids)
-                self.assertTrue('cr1.8xlarge' in ids)
-                self.assertTrue('x1.32xlarge' in ids)
-            elif region_name == 'us-west-1':
-                self.assertEqual(len(sizes), 61)
-            if region_name == 'us-west-2':
-                self.assertEqual(len(sizes), 71)
-            elif region_name == 'ap-southeast-1':
-                self.assertEqual(len(sizes), 57)
-            elif region_name == 'ap-southeast-2':
-                self.assertEqual(len(sizes), 61)
-            elif region_name == 'eu-west-1':
-                self.assertEqual(len(sizes), 68)
-            elif region_name == 'ap-south-1':
-                self.assertEqual(len(sizes), 41)
-
+            self.assertNotEqual(len(sizes), 0)
         self.driver.region_name = region_old
 
     def test_ex_create_node_with_ex_iam_profile(self):
@@ -482,9 +456,9 @@ class EC2Tests(LibcloudTestCase, TestCaseMixin):
         node1 = self.driver.create_node(name='foo', image=image, size=size)
         EC2MockHttp.type = 'ex_iam_profile'
         node2 = self.driver.create_node(name='bar', image=image, size=size,
-                                        ex_iam_profile=iamProfile['name'])
+                                        ex_iamprofile=iamProfile['name'])
         node3 = self.driver.create_node(name='bar', image=image, size=size,
-                                        ex_iam_profile=iamProfile['arn'])
+                                        ex_iamprofile=iamProfile['arn'])
 
         self.assertFalse(node1.extra['iam_profile'])
         self.assertEqual(node2.extra['iam_profile'], iamProfile['id'])
@@ -700,7 +674,7 @@ class EC2Tests(LibcloudTestCase, TestCaseMixin):
 
     def test_import_key_pair_from_string(self):
         path = os.path.join(os.path.dirname(__file__), 'fixtures', 'misc',
-                            'dummy_rsa.pub')
+                            'test_rsa.pub')
 
         with open(path, 'r') as fp:
             key_material = fp.read()
@@ -718,7 +692,7 @@ class EC2Tests(LibcloudTestCase, TestCaseMixin):
 
     def test_import_key_pair_from_file(self):
         path = os.path.join(os.path.dirname(__file__), 'fixtures', 'misc',
-                            'dummy_rsa.pub')
+                            'test_rsa.pub')
 
         key = self.driver.import_key_pair_from_file('keypair', path)
         self.assertEqual(key.name, 'keypair')
@@ -896,6 +870,13 @@ class EC2Tests(LibcloudTestCase, TestCaseMixin):
         self.assertEqual('snap-30d37269', volumes[2].extra['snapshot_id'])
         self.assertEqual(StorageVolumeState.UNKNOWN, volumes[2].state)
 
+        EC2MockHttp.type = 'filters_nodes'
+        node = Node('i-d334b4b3', None, None, None, None, self.driver)
+        self.driver.list_volumes(node=node)
+
+        EC2MockHttp.type = 'filters_status'
+        self.driver.list_volumes(ex_filters={'status': 'available'})
+
     def test_create_volume(self):
         location = self.driver.list_locations()[0]
         vol = self.driver.create_volume(10, 'vol', location)
@@ -904,6 +885,20 @@ class EC2Tests(LibcloudTestCase, TestCaseMixin):
         self.assertEqual('vol', vol.name)
         self.assertEqual('creating', vol.extra['state'])
         self.assertTrue(isinstance(vol.extra['create_time'], datetime))
+        self.assertEqual(False, vol.extra['encrypted'])
+
+    def test_create_encrypted_volume(self):
+        location = self.driver.list_locations()[0]
+        vol = self.driver.create_volume(
+            10, 'vol', location,
+            ex_encrypted=True,
+            ex_kms_key_id='1234')
+
+        self.assertEqual(10, vol.size)
+        self.assertEqual('vol', vol.name)
+        self.assertEqual('creating', vol.extra['state'])
+        self.assertTrue(isinstance(vol.extra['create_time'], datetime))
+        self.assertEqual(True, vol.extra['encrypted'])
 
     def test_destroy_volume(self):
         vol = StorageVolume(id='vol-4282672b', name='test',
@@ -946,6 +941,14 @@ class EC2Tests(LibcloudTestCase, TestCaseMixin):
         # 2013-08-15T16:22:30.000Z
         self.assertEqual(datetime(2013, 8, 15, 16, 22, 30, tzinfo=UTC), snap.created)
 
+    def test_create_volume_snapshot_with_tags(self):
+        vol = StorageVolume(id='vol-4282672b', name='test',
+                            state=StorageVolumeState.AVAILABLE,
+                            size=10, driver=self.driver)
+        snap = self.driver.create_volume_snapshot(
+            vol, 'Test snapshot', ex_metadata={'my_tag': 'test'})
+        self.assertEqual('test', snap.extra['tags']['my_tag'])
+
     def test_list_snapshots(self):
         snaps = self.driver.list_snapshots()
 
@@ -1111,8 +1114,11 @@ class EC2Tests(LibcloudTestCase, TestCaseMixin):
 
     def test_ex_list_networks_filters(self):
         EC2MockHttp.type = 'filters'
-        filters = {'dhcp-options-id': 'dopt-7eded312',  # matches two networks
-                   'cidr': '192.168.51.0/24'}  # matches one network
+
+        filters = OrderedDict([
+            ('dhcp-options-id', 'dopt-7eded312'),  # matches two networks
+            ('cidr', '192.168.51.0/24')  # matches two networks
+        ])
 
         # We assert in the mock http method
         self.driver.ex_list_networks(filters=filters)
@@ -1156,6 +1162,24 @@ class EC2Tests(LibcloudTestCase, TestCaseMixin):
         self.assertEqual('pending', subnet.state)
         self.assertEqual('vpc-532135d1', subnet.extra['vpc_id'])
 
+    def test_ex_modify_subnet_attribute(self):
+        subnet = self.driver.ex_list_subnets()[0]
+        resp = self.driver.ex_modify_subnet_attribute(subnet,
+                                                      'auto_public_ip',
+                                                      True)
+        self.assertTrue(resp)
+        resp = self.driver.ex_modify_subnet_attribute(subnet,
+                                                      'auto_ipv6',
+                                                      False)
+        self.assertTrue(resp)
+
+        expected_msg = 'Unsupported attribute: invalid'
+        self.assertRaisesRegexp(ValueError, expected_msg,
+                                self.driver.ex_modify_subnet_attribute,
+                                subnet,
+                                'invalid',
+                                True)
+
     def test_ex_delete_subnet(self):
         subnet = self.driver.ex_list_subnets()[0]
         resp = self.driver.ex_delete_subnet(subnet=subnet)
@@ -1289,6 +1313,23 @@ class EC2Tests(LibcloudTestCase, TestCaseMixin):
         self.assertEqual('io1', modifications[1].target_volume_type)
         self.assertEqual('vol-bEXAMPLE', modifications[1].volume_id)
 
+    def test_params_is_not_simple_type_exception_is_thrown(self):
+        params = {
+            'not': {'not': ['simple']}
+        }
+
+        expected_msg = 'dictionary contains an attribute "not" which value'
+        self.assertRaisesRegexp(ValueError, expected_msg,
+                               self.driver.connection.request, '/', params=params)
+
+        params = {
+            'invalid': [1, 2, 3]
+        }
+
+        expected_msg = 'dictionary contains an attribute "invalid" which value'
+        self.assertRaisesRegexp(ValueError, expected_msg,
+                               self.driver.connection.request, '/', params=params)
+
 
 class EC2USWest1Tests(EC2Tests):
     region = 'us-west-1'
@@ -1528,7 +1569,13 @@ class EC2MockHttp(MockHttp):
         return (httplib.OK, body, {}, httplib.responses[httplib.OK])
 
     def _CreateVolume(self, method, url, body, headers):
-        body = self.fixtures.load('create_volume.xml')
+        if 'KmsKeyId=' in url:
+            assert 'Encrypted=1' in url, "If a KmsKeyId is specified, the " \
+                                         "Encrypted flag must also be set."
+        if 'Encrypted=1' in url:
+            body = self.fixtures.load('create_encrypted_volume.xml')
+        else:
+            body = self.fixtures.load('create_volume.xml')
         return (httplib.OK, body, {}, httplib.responses[httplib.OK])
 
     def _DeleteVolume(self, method, url, body, headers):
@@ -1547,6 +1594,28 @@ class EC2MockHttp(MockHttp):
         body = self.fixtures.load('describe_volumes.xml')
         return (httplib.OK, body, {}, httplib.responses[httplib.OK])
 
+    def _filters_nodes_DescribeVolumes(self, method, url, body, headers):
+        expected_params = {
+            'Filter.1.Name': 'attachment.instance-id',
+            'Filter.1.Value.1': 'i-d334b4b3',
+        }
+
+        self.assertUrlContainsQueryParams(url, expected_params)
+
+        body = self.fixtures.load('describe_volumes.xml')
+        return (httplib.OK, body, {}, httplib.responses[httplib.OK])
+
+    def _filters_status_DescribeVolumes(self, method, url, body, headers):
+        expected_params = {
+            'Filter.1.Name': 'status',
+            'Filter.1.Value.1': 'available'
+        }
+
+        self.assertUrlContainsQueryParams(url, expected_params)
+
+        body = self.fixtures.load('describe_volumes.xml')
+        return (httplib.OK, body, {}, httplib.responses[httplib.OK])
+
     def _CreateSnapshot(self, method, url, body, headers):
         body = self.fixtures.load('create_snapshot.xml')
         return (httplib.OK, body, {}, httplib.responses[httplib.OK])
@@ -1603,25 +1672,14 @@ class EC2MockHttp(MockHttp):
         return (httplib.OK, body, {}, httplib.responses[httplib.OK])
 
     def _filters_DescribeVpcs(self, method, url, body, headers):
-        expected_params_1 = {
+        expected_params = {
             'Filter.1.Name': 'dhcp-options-id',
             'Filter.1.Value.1': 'dopt-7eded312',
             'Filter.2.Name': 'cidr',
             'Filter.2.Value.1': '192.168.51.0/24'
         }
 
-        expected_params_2 = {
-            'Filter.1.Name': 'cidr',
-            'Filter.1.Value.1': '192.168.51.0/24',
-            'Filter.2.Name': 'dhcp-options-id',
-            'Filter.2.Value.1': 'dopt-7eded312'
-        }
-
-        try:
-            self.assertUrlContainsQueryParams(url, expected_params_1)
-        except AssertionError:
-            # dict ordering is not guaranteed
-            self.assertUrlContainsQueryParams(url, expected_params_2)
+        self.assertUrlContainsQueryParams(url, expected_params)
 
         body = self.fixtures.load('describe_vpcs.xml')
         return (httplib.OK, body, {}, httplib.responses[httplib.OK])
@@ -1642,6 +1700,10 @@ class EC2MockHttp(MockHttp):
         body = self.fixtures.load('create_subnet.xml')
         return (httplib.OK, body, {}, httplib.responses[httplib.OK])
 
+    def _ModifySubnetAttribute(self, method, url, body, headers):
+        body = self.fixtures.load('modify_subnet_attribute.xml')
+        return (httplib.OK, body, {}, httplib.responses[httplib.OK])
+
     def _DeleteSubnet(self, method, url, body, headers):
         body = self.fixtures.load('delete_subnet.xml')
         return (httplib.OK, body, {}, httplib.responses[httplib.OK])
@@ -1801,6 +1863,14 @@ class NimbusTests(EC2Tests):
         self.driver.ex_create_tags(resource=node, tags={'foo': 'bar'})
         self.assertExecutedMethodCount(0)
 
+    def test_create_volume_snapshot_with_tags(self):
+        vol = StorageVolume(id='vol-4282672b', name='test',
+                            state=StorageVolumeState.AVAILABLE,
+                            size=10, driver=self.driver)
+        snap = self.driver.create_volume_snapshot(
+            vol, 'Test snapshot', ex_metadata={'my_tag': 'test'})
+        self.assertDictEqual({}, snap.extra['tags'])
+
 
 class EucTests(LibcloudTestCase, TestCaseMixin):
 
@@ -1922,6 +1992,18 @@ class OutscaleTests(EC2Tests):
         self.assertTrue('m1.large' in ids)
         self.assertTrue('m1.xlarge' in ids)
 
+    def test_ex_create_node_with_ex_iam_profile(self):
+        image = NodeImage(id='ami-be3adfd7',
+                          name=self.image_name,
+                          driver=self.driver)
+        size = NodeSize('m1.small', 'Small Instance', None, None, None, None,
+                        driver=self.driver)
+
+        self.assertRaises(NotImplementedError, self.driver.create_node,
+                          name='foo',
+                          image=image, size=size,
+                          ex_iamprofile='foo')
+
 
 class FCUMockHttp(EC2MockHttp):
     fixtures = ComputeFileFixtures('fcu')
