@@ -21,14 +21,18 @@ import sys
 from io import BytesIO
 from hashlib import sha1
 
+import mock
 from mock import Mock
 from mock import PropertyMock
+import libcloud.utils.files  # NOQA: F401
 
 from libcloud.utils.py3 import ET
 from libcloud.utils.py3 import httplib
 from libcloud.utils.py3 import urlparse
 from libcloud.utils.py3 import parse_qs
 from libcloud.utils.py3 import StringIO
+from libcloud.utils.py3 import PY3
+from libcloud.utils.files import exhaust_iterator
 
 from libcloud.common.types import InvalidCredsError
 from libcloud.common.types import LibcloudError, MalformedResponseError
@@ -41,9 +45,6 @@ from libcloud.storage.types import ObjectDoesNotExistError
 from libcloud.storage.types import ObjectHashMismatchError
 from libcloud.storage.drivers.s3 import BaseS3Connection
 from libcloud.storage.drivers.s3 import S3StorageDriver, S3USWestStorageDriver
-from libcloud.storage.drivers.s3 import S3EUWestStorageDriver
-from libcloud.storage.drivers.s3 import S3APSEStorageDriver
-from libcloud.storage.drivers.s3 import S3APNEStorageDriver
 from libcloud.storage.drivers.s3 import CHUNK_SIZE
 from libcloud.utils.py3 import b
 
@@ -312,7 +313,7 @@ class S3MockHttp(MockHttp):
             except ValueError:
                 # lxml wants a bytes and tests are basically hard-coded to str
                 body = ET.XML(self.body.encode('utf-8'))
-        except:
+        except Exception:
             raise MalformedResponseError("Failed to parse XML",
                                          body=self.body,
                                          driver=self.connection.driver)
@@ -404,8 +405,7 @@ class S3Tests(unittest.TestCase):
         self.mock_response_klass.type = 'UNAUTHORIZED'
         try:
             self.driver.list_containers()
-        except InvalidCredsError:
-            e = sys.exc_info()[1]
+        except InvalidCredsError as e:
             self.assertEqual(True, isinstance(e, InvalidCredsError))
         else:
             self.fail('Exception was not thrown')
@@ -638,7 +638,6 @@ class S3Tests(unittest.TestCase):
 
         # If content is consumed and response.content attribute accessed execption
         # will be thrown and test will fail
-
         mock_response = Mock(name='mock response')
         mock_response.headers = {}
         mock_response.status_code = 200
@@ -662,6 +661,40 @@ class S3Tests(unittest.TestCase):
                                              delete_on_failure=True)
         self.assertTrue(result)
 
+    def test_download_object_as_stream_data_is_not_buffered_in_memory(self):
+        # Test case which verifies that response.response attribute is not accessed
+        # and as such, whole body response is not buffered into RAM
+
+        # If content is consumed and response.content attribute accessed exception
+        # will be thrown and test will fail
+        mock_response = Mock(name='mock response')
+        mock_response.headers = {}
+        mock_response.status = 200
+
+        msg1 = '"response" attribute was accessed but it shouldn\'t have been'
+        msg2 = '"content" attribute was accessed but it shouldn\'t have been'
+        type(mock_response).response = PropertyMock(name='mock response attribute',
+                                                    side_effect=Exception(msg1))
+        type(mock_response).content = PropertyMock(name='mock content attribute',
+                                                   side_effect=Exception(msg2))
+        mock_response.iter_content.return_value = StringIO('a' * 1000)
+
+        self.driver.connection.request = Mock()
+        self.driver.connection.request.return_value = mock_response
+
+        container = Container(name='foo_bar_container', extra={},
+                              driver=self.driver)
+        obj = Object(name='foo_bar_object_NO_BUFFER', size=1000, hash=None, extra={},
+                     container=container, meta_data=None,
+                     driver=self.driver_type)
+        result = self.driver.download_object_as_stream(obj=obj)
+        result = exhaust_iterator(result)
+
+        if PY3:
+            result = result.decode('utf-8')
+
+        self.assertEqual(result, 'a' * 1000)
+
     def test_download_object_invalid_file_size(self):
         self.mock_response_klass.type = 'INVALID_SIZE'
         container = Container(name='foo_bar_container', extra={},
@@ -728,8 +761,7 @@ class S3Tests(unittest.TestCase):
                                       object_name=object_name,
                                       verify_hash=True,
                                       ex_storage_class='invalid-class')
-        except ValueError:
-            e = sys.exc_info()[1]
+        except ValueError as e:
             self.assertTrue(str(e).lower().find('invalid storage class') != -1)
         else:
             self.fail('Exception was not thrown')
@@ -825,7 +857,8 @@ class S3Tests(unittest.TestCase):
         def upload_file(self, object_name=None, content_type=None,
                         request_path=None, request_method=None,
                         headers=None, file_path=None, stream=None):
-            return {'response': make_response(200, headers={'etag': '0cc175b9c0f1b6a831c399e269772661'}),
+            headers = {'etag': '0cc175b9c0f1b6a831c399e269772661'}
+            return {'response': make_response(200, headers=headers),
                     'bytes_transferred': 1000,
                     'data_hash': '0cc175b9c0f1b6a831c399e269772661'}
 
@@ -905,6 +938,27 @@ class S3Tests(unittest.TestCase):
         self.assertEqual(obj.name, object_name)
         self.assertEqual(obj.size, CHUNK_SIZE * 3)
 
+    def test_upload_object_via_stream_guess_file_mime_type(self):
+        if self.driver.supports_s3_multipart_upload:
+            self.mock_response_klass.type = 'MULTIPART'
+        else:
+            self.mock_response_klass.type = None
+
+        container = Container(name='foo_bar_container', extra={},
+                              driver=self.driver)
+        object_name = 'foo_test_stream_data'
+        iterator = BytesIO(b('234'))
+
+        with mock.patch(
+                'libcloud.utils.files.guess_file_mime_type',
+                autospec=True) as mock_guess_file_mime_type:
+            mock_guess_file_mime_type.return_value = ('application/zip', None)
+
+            self.driver.upload_object_via_stream(container=container,
+                                                 object_name=object_name,
+                                                 iterator=iterator)
+            mock_guess_file_mime_type.assert_called_with(object_name)
+
     def test_upload_object_via_stream_abort(self):
         if not self.driver.supports_s3_multipart_upload:
             return
@@ -984,21 +1038,50 @@ class S3Tests(unittest.TestCase):
         result = self.driver.delete_object(obj=obj)
         self.assertTrue(result)
 
+    def test_region_keyword_argument(self):
+        # Default region
+        driver  = S3StorageDriver(*self.driver_args)
+        self.assertEqual(driver.region, 'us-east-1')
+        self.assertEqual(driver.connection.host, 's3.amazonaws.com')
 
-class S3USWestTests(S3Tests):
-    driver_type = S3USWestStorageDriver
+        # Custom region
+        driver  = S3StorageDriver(*self.driver_args, region='us-west-2')
+        self.assertEqual(driver.region, 'us-west-2')
+        self.assertEqual(driver.connection.host, 's3-us-west-2.amazonaws.com')
 
+        # Verify class instance and class variables don't get mixed up
+        driver1  = S3StorageDriver(*self.driver_args, region='us-west-2')
+        self.assertEqual(driver1.region, 'us-west-2')
+        self.assertEqual(driver1.connection.host, 's3-us-west-2.amazonaws.com')
 
-class S3EUWestTests(S3Tests):
-    driver_type = S3EUWestStorageDriver
+        driver2  = S3StorageDriver(*self.driver_args, region='ap-south-1')
+        self.assertEqual(driver2.region, 'ap-south-1')
+        self.assertEqual(driver2.connection.host, 's3-ap-south-1.amazonaws.com')
 
+        self.assertEqual(driver1.region, 'us-west-2')
+        self.assertEqual(driver1.connection.host, 's3-us-west-2.amazonaws.com')
 
-class S3APSETests(S3Tests):
-    driver_type = S3APSEStorageDriver
+        # Test all supported regions
+        for region in S3StorageDriver.list_regions():
+            driver = S3StorageDriver(*self.driver_args, region=region)
+            self.assertEqual(driver.region, region)
 
+        # Invalid region
+        expected_msg = 'Invalid or unsupported region: foo'
+        self.assertRaisesRegexp(ValueError, expected_msg, S3StorageDriver,
+                                *self.driver_args, region='foo')
 
-class S3APNETests(S3Tests):
-    driver_type = S3APNEStorageDriver
+        # host argument still has precedence over reguin
+        driver3  = S3StorageDriver(*self.driver_args, region='ap-south-1', host='host1.bar.com')
+        self.assertEqual(driver3.region, 'ap-south-1')
+        self.assertEqual(driver3.connection.host, 'host1.bar.com')
+
+        driver4  = S3StorageDriver(*self.driver_args, host='host2.bar.com')
+        self.assertEqual(driver4.connection.host, 'host2.bar.com')
+
+    def test_deprecated_driver_class_per_region(self):
+        driver = S3USWestStorageDriver(*self.driver_args)
+        self.assertEqual(driver.region, 'us-west-1')
 
 
 if __name__ == '__main__':
